/**
 * Main API
 * @module EvamApi
 */
import { Battery, DeviceRole, DisplayMode, GRPC, InternetState, Location, Notification, Operation, TripLocationHistory, VehicleState, VehicleStatus, RakelState } from "../domain";
import { LayerPointData, LayerShapeData } from "../domain/LayerData";
import { RawRakelAction } from "../domain/RawRakelAction";
declare type CallbackFunction<T1, T2 = void> = (t: T1) => T2;
/**
 * Evam API singleton that exposes methods to interact with the Evam platform.
 *
 * @example
 * ```ts
 * // Get instance (don't be afraid to copy them around or create more, as they're simply a lightweight reference to shared static data)
 * const evamApi = new EvamApi();
 *
 * // Register a new callback on any operation update that simply logs it
 * evamApi.onNewOrUpdatedActiveOperation((activeOperation) => console.log(activeOperation));
 *
 * // Register a new callback on any application settings update that simply logs them
 * evamApi.onNewOrUpdatedSettings((settings) => console.log(settings));
 *
 * //Register a new callback to any specific application data update. Available register methods:
 * evamApi.onNewOrUpdatedDeviceRole((deviceRole) => ...);
 * evamApi.onNewOrUpdatedLocation((location) => ...);
 * evamApi.onNewOrUpdatedInternetState((internetState) => ...);
 * evamApi.onNewOrUpdatedVehicleState((vehicleState) => ...);
 * evamApi.onNewOrUpdatedTripLocationHistory((tripLocationHistory) => ...);
 *
 * // Send a new notification to VS (this will also work for the developer environment)
 * // This will display a notification with heading "example", description "lorem ipsum". It will have type 'QUICK_HUN' and a primary button
 * // which is labelled 'primary button' that doesn't do anything when called. The secondary button is not defined, thus it will not display.
 * evamApi.sendNotification(new Notification("example","lorem ipsum", NotificationType.QUICK_HUN, {label:'primary button', callback:()=>{}}, undefined))
 *
 * //Remove all callbacks from the SDK (this is useful for cleanup)
 * evamApi.unsubscribeFromAllCallbacks()
 *
 * //Detect whether the certified app is currently running in Vehicle Services
 * EvamApi.isRunningInVehicleServices
 *
 * //update the current hospital by id (be sure that the hospital is listed in available hospitals)
 * evamApi.setHospital(1234)
 *
 * //update the current priority (be sure that the priority is listed in available priorities)
 * evamApi.setPriority (1)
 *
 * //simulate Vehicle Services data inject (development + testing only)
 * //DO NOT USE THESE METHODS IN PRODUCTION, While not breaking by any means they will not perform any function.
 * evamApi.injectLocation(new Location(59.364, 18.012, new Date()))
 * evamApi.injectVehicleState(new VehicleState(...))
 * evamApi.injectTrip(new TripLocationHistory(...))
 * evamApi.injectDeviceRole(new DeviceRole(...))
 * evamApi.injectInternetState(new InternetState(...))
 * evamApi.injectOperation(new Operation(...))
 * evamApi.injectSettings(new Settings(...))
 * evamApi.injectOperationList([new Operation(...), new Operation(...), ...])
 * evamApi.injectBattery(new Battery(...)))
 *```
 *
 */
export declare class EvamApi {
    private static singletonExists;
    static isRunningInVehicleServices: boolean;
    constructor();
    /**
     * EvamData instance for storing data regard Vehicle Services.
     * @private
     */
    private static evamData;
    /**
     * These arrays store references to every callback subscribed to certain events. We use these references to later unsubscribe from DOM events.
     * @hidden
     */
    private static newOrUpdatedOperationCallbacks;
    private static newOrUpdatedSettingsCallbacks;
    private static newOrUpdatedLocationCallbacks;
    private static newOrUpdatedDeviceRoleCallbacks;
    private static newOrUpdatedInternetStateCallbacks;
    private static newOrUpdatedVehicleStateCallbacks;
    private static newOrUpdatedTripLocationHistoryCallbacks;
    private static newOrUpdatedOperationListCallbacks;
    private static newOrUpdatedBatteryCallbacks;
    private static newOrUpdatedDisplayModeCallbacks;
    private static newOrUpdatedRakelStateCallbacks;
    private static newOrUpdatedAvailableVehicleStatusList;
    private static newOrUpdatedRakelMessages;
    private static notificationCallbacks;
    /**
     * The persistentStorageMap object is only used for testing. It is null unless the isRunningInVehicleServices method
     * returns false.
     */
    private static persistentStorageMap;
    /**
     * Unsubscribes all registered callbacks from Vehicle Service events.
     */
    unsubscribeFromAllCallbacks: () => void;
    /**
     * Store used for persisting data within Vehicle Services. If you are in development and not running your application within vehicle services then this
     * will serve as a lightweight wrapper around localstorage. To avoid naming conflicts AppId must be set as it is used to identify application items.
     */
    store: {
        /**
         * Stores an item in Vehicle Services
         * @param key the identifying name of the item
         * @param value the value of the item
         */
        set: (key: string, value: string) => void;
        /**
         * Retrieves an item from Vehicle Services
         * @param key the identifying name of the item
         */
        get: (key: string) => string;
        /**
         * Deletes an item from Vehicle Services
         * @param key the identifying name of the item
         */
        delete: (key: string) => void;
        /**
         * Deletes all item from Vehicle Services
         */
        clear: () => void;
    };
    /**
     * Sets the selected hospital id for the current active operation. The id must be present inside the available hospitals
     * @requires Permissions ACTIVE_OPERATION_WRITE
     * @param id the id of the hospital to be set
     */
    setHospital(id: number): void;
    /**
     * Sets the selected priority id for the current active operation. The id must be present inside the available priorities.
     * @requires Permissions ACTIVE_OPERATION_WRITE
     * @param id of the priority to be set
     */
    setPriority(id: number): void;
    /**
     * Manually inject location to EvamApi (Only available in development.)
     * @param location the location to inject.
     */
    injectLocation(location: Location): void;
    /**
     * Manually inject the Available Vehicle Status list to EvamApi (Only available in development.)
     * @param vehicleStatusList the list of available Vehicle Statuses
     */
    injectAvailableVehicleStatusList(vehicleStatusList: VehicleStatus[]): void;
    /**
     * Manually inject the Rakel State to EvamApi (Only available in development.)
     * @param rakelState The Rakel State
     */
    injectRakelState(rakelState: RakelState): void;
    /**
     * Manually inject vehicleState to EvamApi (Only available in development.)
     * @param vehicleState the vehicleState to inject.
     */
    injectVehicleState(vehicleState: VehicleState): void;
    /**
     * Manually inject tripLocationHistory to EvamApi (Only available in development.)
     * @param tripLocationHistory the tripLocationHistory to inject.
     */
    injectTrip(tripLocationHistory: TripLocationHistory): void;
    /**
     * Manually inject deviceRole to EvamApi (Only available in development.)
     * @param deviceRole the deviceRole to inject.
     */
    injectDeviceRole(deviceRole: DeviceRole): void;
    /**
     * Manually inject internetState to EvamApi (Only available in development.)
     * @param internetState the internetState to inject.
     */
    injectInternetState(internetState: InternetState): void;
    /**
     * Injects the Active Operation manually. This will trigger onNewOrUpdatedActiveOperation(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param activeCase The active case to be injected for development purposes.
     */
    injectOperation(activeCase: Operation): void;
    /**
     * Injects the settings manually. This will trigger onNewOrUpdatedSettings(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param settings The settings to be injected for development purposes.
     */
    injectSettings(settings: any): void;
    /**
     * Injects the display mode manually. This will trigger onNewOrUpdatedDisplayMode(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param displayMode The display mode (light or dark) to be injected for development purposes.
     */
    injectDisplayMode(displayMode: DisplayMode): void;
    injectAppVersion(appVersion: string): void;
    injectOSVersion(osVersion: string): void;
    injectVSVersion(vsVersion: string): void;
    injectAppId(appId: string): void;
    /**
     * Injects the operation list manually. This will trigger onNewOrUpdatedOperationList(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param operationList The operation list to be injected for development purposes.
     */
    injectOperationList(operationList: Operation[]): void;
    /**
     * Injects the battery manually. This will trigger onNewOrUpdatedBattery(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param battery The battery to be injected for development purposes.
     */
    injectBattery(battery: Battery): void;
    /**
     * Inject a list of raw Rakel messages as they would be received from the radio.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param rakelMessages list of raw Rakel messages.
     */
    injectRakelMessages(rakelMessages: string[]): void;
    /**
     * Gets the address for the GRPC proxy
     */
    getGRPC: () => GRPC;
    /**
     * Gets the device ID as defined in Android
     */
    getDeviceId: () => string;
    /**
     * Gets the Evam App id as given by Vehicle Services
     */
    getAppId: () => string;
    /**
     * Gets the Evam App version as defined in the evam.json manifest
     */
    getAppVersion: () => string;
    /**
     * Gets the Android OS version
     */
    getOSVersion: () => string;
    /**
     * Gets the Vehicle Services app version
     */
    getVehicleServicesVersion: () => string;
    /**
     * Registers a callback to be run upon a new Active Operation is available or the current Active
     * Operation is updated.
     * @requires Permissions ACTIVE_OPERATION_READ
     * @param callback The callback to be executed
     */
    onNewOrUpdatedActiveOperation(callback: CallbackFunction<Operation | undefined>): void;
    /**
     * Registers a callback to be run upon new application settings reception or settings update
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedSettings(callback: CallbackFunction<any | undefined>): void;
    /**
     * Registers a callback to be run upon new device role or device role update
     * @requires Permissions DEVICE_ROLE_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedDeviceRole(callback: CallbackFunction<DeviceRole | undefined>): void;
    /**
     * Registers a callback to be run upon new location or location update
     * @requires Permissions LOCATION_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedLocation(callback: CallbackFunction<Location | undefined>): void;
    /**
     * Registers a callback to be run upon new internetState or internetState update
     * @requires Permissions CONNECTIVITY_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedInternetState(callback: CallbackFunction<InternetState | undefined>): void;
    /**
     * Used to assign a callback when the vehicle state is updated.
     * @requires Permissions VEHICLE_STATE_READ
     * @param callback The callback with (optional) argument vehicleState. Use this to access the vehicle state.
     */
    onNewOrUpdatedVehicleState(callback: CallbackFunction<VehicleState | undefined>): void;
    /**
     * Used to assign a callback when the trip location history is updated.
     * @requires Permissions TRIP_HISTORY_READ
     * @param callback The callback with (optional) argument tripLocationHistory. Use this to access the trip location history.
     */
    onNewOrUpdatedTripLocationHistory(callback: CallbackFunction<TripLocationHistory | undefined>): void;
    /**
     * Used to assign a callback when the operation list is updated.
     * @requires Permissions OPERATION_READ
     * @param callback The callback with (optional) argument operationList. Use this to access the operation list.
     */
    onNewOrUpdatedOperationList(callback: CallbackFunction<Operation[] | undefined>): void;
    /**
     * Used to assign a callback when the battery data is updated.
     * @requires Permissions BATTERY_READ
     * @param callback The callback with (optional) argument battery. Use this to access the battery.
     */
    onNewOrUpdatedBattery(callback: CallbackFunction<Battery | undefined>): void;
    /**
     *
     * Used to assign a callback when the battery created or updated.
     * @requires Permissions DISPLAY_MODE_READ
     * @param callback The callback with (optional) argument display mode. Use this to access the display mode.
     */
    onNewOrUpdatedDisplayMode(callback: CallbackFunction<DisplayMode | undefined>): void;
    /**
     * Used to assign a callback when the rakel state is created or updated
     * @requires Permissions RAKEL_STATE_READ
     * @param callback The callback with (optional) argument Rakel state. Use this to access the Rakel state.
     * @preview This function is currently available in the Development Environment only.
     */
    onNewOrUpdatedRakelState(callback: CallbackFunction<RakelState | undefined>): void;
    /**
     * Used to assign a callback when the list of available Vehicle Statuses is created or updated
     * @requires Permissions AVAILABLE_VEHICLE_STATUS_LIST_READ
     * @preview This function is currently available in the Development Environment only.
     * @param callback The callback with (optional) argument available Vehicle Status list. Use this to access the available Vehicle Statuses.
     */
    onNewOrUpdatedAvailableVehicleStatusList(callback: CallbackFunction<VehicleStatus[] | undefined>): void;
    /**
     * Used to assign a callback when the incoming Rakel messages are updated.
     * The messages are piped though in the raw form as they are received from the radio.
     * @requires Permissions RAKEL_COMMUNICATION_READ
     * @preview This function is currently available in the Development Environment only.
     * @param callback The callback with (optional) argument Rakel messages. Use this to access the incoming Rakel messages.
     */
    onNewOrUpdatedRakelMessages(callback: CallbackFunction<string[] | undefined>): void;
    /**
     * send a notification to vehicle services (or evam-dev-environment if using the dev environment)
     * @requires Permissions SEND_NOTIFICATION
     * @param notification The notification to be sent
     */
    sendNotification(notification: Notification): void;
    private static triggerNotificationCallback;
    /**
     * Adds/Update a layer by its ID. Reusing a layerID causes the data to be replaced. A certified app can only update a layer it has created.
     * This function adds a set of points on the map with text and icon at the specified lat and lon
     * @param id the id of the layer (if the layer doesn't exist then one will be created)
     * @param layerData array of points to be shown with text and icon. Note that the icon of the first element will be used for all points.
     * @requires Permissions NAVIGATION_PRIVATE_LAYERS
     * @preview This function is currently available in the Development Environment only.
     */
    setNavLayerPoint: (id: string, layerData: LayerPointData[]) => void;
    /**
     * Adds/Update a layer by its ID. Reusing a layerID causes the data to be replaced. A certified app can only update a layer it has created.
     * This function adds a set of shapes on the map with the text in its center.
     * @param id the id of the layer (if the layer doesn't exist then one will be created)
     * @param layerData array of shapes to be shown with text and shape color (format: "#AARRGGBB", just like the SC buttons)
     * @requires Permissions NAVIGATION_PRIVATE_LAYERS
     * @preview This function is currently available in the Development Environment only.
     */
    setNavLayerShape: (id: string, layerData: LayerShapeData[]) => void;
    /**
     * Deletes a layer by its ID. A certified app can only delete a layer it has created.
     * @param id the id of the layer (if the layer doesn't exist then one will be created)
     * @requires Permissions NAVIGATION_PRIVATE_LAYERS
     * @preview This function is currently available in the Development Environment only.
     */
    deleteNavLayer: (id: string) => void;
    /**
     * Sends a RawRakelAction to the Rakel radio.
     * @param rawRakelAction the RawRakelAction to be sent to the radio.
     * @requires Permissions RAKEL_RAW_COMMAND_SEND
     */
    sendRawRakelAction: (rawRakelAction: RawRakelAction) => void;
}
export {};
