"use strict";
/**
 * Main API
 * @module EvamApi
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvamApi = void 0;
const domain_1 = require("../domain");
const EventHelpers_1 = require("../util/EventHelpers");
const uuid_1 = require("uuid");
const lodash_1 = __importDefault(require("lodash"));
const AndroidNativeHelpers_1 = require("./AndroidNativeHelpers");
/**
 * @hidden
 */
class EvamData {
    constructor(activeCase, settings, internetState, deviceRole, location, vehicleState, tripLocationHistory, operationList, battery, osVersion, vsVersion, appVersion, deviceId, displayMode, grpc, appId, rakelState, availableVehicleStatusList, rakelMessages) {
        this.activeCase = activeCase;
        this.settings = settings;
        this.internetState = internetState;
        this.deviceRole = deviceRole;
        this.location = location;
        this.vehicleState = vehicleState;
        this.tripLocationHistory = tripLocationHistory;
        this.operationList = operationList;
        this.battery = battery;
        this.osVersion = osVersion;
        this.vsVersion = vsVersion;
        this.appVersion = appVersion;
        this.deviceId = deviceId;
        this.displayMode = displayMode;
        this.grpc = grpc;
        this.appId = appId;
        this.rakelState = rakelState;
        this.availableVehicleStatusList = availableVehicleStatusList;
        this.rakelMessages = rakelMessages;
    }
}
const vsLog = (...args) => {
    if (EvamApi.isRunningInVehicleServices) {
        console.log(args);
    }
};
/**
 * Evam API singleton that exposes methods to interact with the Evam platform.
 *
 * @example
 * ```ts
 * // Get instance (don't be afraid to copy them around or create more, as they're simply a lightweight reference to shared static data)
 * const evamApi = new EvamApi();
 *
 * // Register a new callback on any operation update that simply logs it
 * evamApi.onNewOrUpdatedActiveOperation((activeOperation) => console.log(activeOperation));
 *
 * // Register a new callback on any application settings update that simply logs them
 * evamApi.onNewOrUpdatedSettings((settings) => console.log(settings));
 *
 * //Register a new callback to any specific application data update. Available register methods:
 * evamApi.onNewOrUpdatedDeviceRole((deviceRole) => ...);
 * evamApi.onNewOrUpdatedLocation((location) => ...);
 * evamApi.onNewOrUpdatedInternetState((internetState) => ...);
 * evamApi.onNewOrUpdatedVehicleState((vehicleState) => ...);
 * evamApi.onNewOrUpdatedTripLocationHistory((tripLocationHistory) => ...);
 *
 * // Send a new notification to VS (this will also work for the developer environment)
 * // This will display a notification with heading "example", description "lorem ipsum". It will have type 'QUICK_HUN' and a primary button
 * // which is labelled 'primary button' that doesn't do anything when called. The secondary button is not defined, thus it will not display.
 * evamApi.sendNotification(new Notification("example","lorem ipsum", NotificationType.QUICK_HUN, {label:'primary button', callback:()=>{}}, undefined))
 *
 * //Remove all callbacks from the SDK (this is useful for cleanup)
 * evamApi.unsubscribeFromAllCallbacks()
 *
 * //Detect whether the certified app is currently running in Vehicle Services
 * EvamApi.isRunningInVehicleServices
 *
 * //update the current hospital by id (be sure that the hospital is listed in available hospitals)
 * evamApi.setHospital(1234)
 *
 * //update the current priority (be sure that the priority is listed in available priorities)
 * evamApi.setPriority (1)
 *
 * //simulate Vehicle Services data inject (development + testing only)
 * //DO NOT USE THESE METHODS IN PRODUCTION, While not breaking by any means they will not perform any function.
 * evamApi.injectLocation(new Location(59.364, 18.012, new Date()))
 * evamApi.injectVehicleState(new VehicleState(...))
 * evamApi.injectTrip(new TripLocationHistory(...))
 * evamApi.injectDeviceRole(new DeviceRole(...))
 * evamApi.injectInternetState(new InternetState(...))
 * evamApi.injectOperation(new Operation(...))
 * evamApi.injectSettings(new Settings(...))
 * evamApi.injectOperationList([new Operation(...), new Operation(...), ...])
 * evamApi.injectBattery(new Battery(...)))
 *```
 *
 */
class EvamApi {
    constructor() {
        /**
         * Unsubscribes all registered callbacks from Vehicle Service events.
         */
        this.unsubscribeFromAllCallbacks = () => {
            const clearCallbacksAndArray = (callbackFunctionArray, event) => {
                callbackFunctionArray.forEach((callback) => {
                    (0, EventHelpers_1.unsubscribe)(event, callback);
                });
                //This empties the array.
                //The reason I don't just do 'array = []' here is because TypeScript throws a warning, which will mess with pipelines.
                callbackFunctionArray.splice(0, callbackFunctionArray.length);
            };
            clearCallbacksAndArray(EvamApi.newOrUpdatedOperationCallbacks, domain_1.EvamEvent.NewOrUpdatedOperation);
            clearCallbacksAndArray(EvamApi.newOrUpdatedSettingsCallbacks, domain_1.EvamEvent.NewOrUpdatedSettings);
            clearCallbacksAndArray(EvamApi.newOrUpdatedLocationCallbacks, domain_1.EvamEvent.NewOrUpdatedLocation);
            clearCallbacksAndArray(EvamApi.newOrUpdatedDeviceRoleCallbacks, domain_1.EvamEvent.NewOrUpdatedDeviceRole);
            clearCallbacksAndArray(EvamApi.newOrUpdatedInternetStateCallbacks, domain_1.EvamEvent.NewOrUpdatedInternetState);
            clearCallbacksAndArray(EvamApi.newOrUpdatedVehicleStateCallbacks, domain_1.EvamEvent.NewOrUpdatedVehicleState);
            clearCallbacksAndArray(EvamApi.newOrUpdatedTripLocationHistoryCallbacks, domain_1.EvamEvent.NewOrUpdatedTripLocationHistory);
            clearCallbacksAndArray(EvamApi.newOrUpdatedBatteryCallbacks, domain_1.EvamEvent.NewOrUpdatedBattery);
            clearCallbacksAndArray(EvamApi.newOrUpdatedDisplayModeCallbacks, domain_1.EvamEvent.NewOrUpdatedDisplayMode);
            clearCallbacksAndArray(EvamApi.newOrUpdatedRakelStateCallbacks, domain_1.EvamEvent.NewOrUpdateRakelState);
            clearCallbacksAndArray(EvamApi.newOrUpdatedAvailableVehicleStatusList, domain_1.EvamEvent.NewOrUpdatedAvailableVehicleStatusList);
            clearCallbacksAndArray(EvamApi.newOrUpdatedRakelMessages, domain_1.EvamEvent.NewOrUpdatedRakelMessages);
            EvamApi.notificationCallbacks.clear();
        };
        /**
         * Store used for persisting data within Vehicle Services. If you are in development and not running your application within vehicle services then this
         * will serve as a lightweight wrapper around localstorage. To avoid naming conflicts AppId must be set as it is used to identify application items.
         */
        this.store = {
            /**
             * Stores an item in Vehicle Services
             * @param key the identifying name of the item
             * @param value the value of the item
             */
            set: (key, value) => {
                if (EvamApi.isRunningInVehicleServices) {
                    (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).setItem(key, value);
                }
                else {
                    if (EvamApi.persistentStorageMap !== null) {
                        EvamApi.persistentStorageMap.set(key, value);
                        if (this.getAppId() === undefined) {
                            console.warn("Using EvamApi localstorage functions will not persist until you set the app id. If you are not running in Vehicle Services then you need to call");
                            return;
                        }
                        localStorage.setItem(this.getAppId() + key, value);
                    }
                }
            },
            /**
             * Retrieves an item from Vehicle Services
             * @param key the identifying name of the item
             */
            get: (key) => {
                if (EvamApi.isRunningInVehicleServices) {
                    return (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).getItem(key);
                }
                else {
                    if (EvamApi.persistentStorageMap !== null) {
                        if (this.getAppId() === undefined) {
                            return EvamApi.persistentStorageMap.get(key);
                        }
                        return localStorage.getItem(this.getAppId() + key);
                    }
                }
            },
            /**
             * Deletes an item from Vehicle Services
             * @param key the identifying name of the item
             */
            delete: (key) => {
                if (EvamApi.isRunningInVehicleServices) {
                    (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).deleteItem(key);
                }
                else {
                    if (EvamApi.persistentStorageMap !== null) {
                        EvamApi.persistentStorageMap.delete(key);
                        if (this.getAppId() === undefined) {
                            console.warn("Using EvamApi localstorage functions will not persist until you set the app id. If you are not running in Vehicle Services then you need to call");
                            return;
                        }
                        localStorage.removeItem(this.getAppId() + key);
                    }
                }
            },
            /**
             * Deletes all item from Vehicle Services
             */
            clear: () => {
                if (EvamApi.isRunningInVehicleServices) {
                    (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).clearItems();
                }
                else {
                    if (EvamApi.persistentStorageMap !== null) {
                        EvamApi.persistentStorageMap.clear();
                        if (this.getAppId() === undefined) {
                            console.warn("Using EvamApi localstorage functions will not persist until you set the app id. If you are not running in Vehicle Services then you need to call");
                            return;
                        }
                        localStorage.clear();
                    }
                }
            }
        };
        /**
         * Gets the address for the GRPC proxy
         */
        this.getGRPC = () => EvamApi.evamData.grpc;
        /**
         * Gets the device ID as defined in Android
         */
        this.getDeviceId = () => EvamApi.evamData.deviceId;
        /**
         * Gets the Evam App id as given by Vehicle Services
         */
        this.getAppId = () => EvamApi.evamData.appId;
        //These get*Version functions are different from the other ways of getting data from the SDK.
        //The software versions are set once and then not changed again, so it's fine to allow the developer to get these whenever they want.
        /**
         * Gets the Evam App version as defined in the evam.json manifest
         */
        this.getAppVersion = () => EvamApi.evamData.appVersion;
        /**
         * Gets the Android OS version
         */
        this.getOSVersion = () => EvamApi.evamData.osVersion;
        /**
         * Gets the Vehicle Services app version
         */
        this.getVehicleServicesVersion = () => EvamApi.evamData.vsVersion;
        /**
         * Adds/Update a layer by its ID. Reusing a layerID causes the data to be replaced. A certified app can only update a layer it has created.
         * This function adds a set of points on the map with text and icon at the specified lat and lon
         * @param id the id of the layer (if the layer doesn't exist then one will be created)
         * @param layerData array of points to be shown with text and icon. Note that the icon of the first element will be used for all points.
         * @requires Permissions NAVIGATION_PRIVATE_LAYERS
         * @preview This function is currently available in the Development Environment only.
         */
        this.setNavLayerPoint = (id, layerData) => {
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NavLayerPointSet, {
                id,
                layerData
            });
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).setNavLayerPoint(id, layerData);
        };
        /**
         * Adds/Update a layer by its ID. Reusing a layerID causes the data to be replaced. A certified app can only update a layer it has created.
         * This function adds a set of shapes on the map with the text in its center.
         * @param id the id of the layer (if the layer doesn't exist then one will be created)
         * @param layerData array of shapes to be shown with text and shape color (format: "#AARRGGBB", just like the SC buttons)
         * @requires Permissions NAVIGATION_PRIVATE_LAYERS
         * @preview This function is currently available in the Development Environment only.
         */
        this.setNavLayerShape = (id, layerData) => {
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NavLayerShapeSet, {
                id,
                layerData
            });
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).setNavLayerShape(id, layerData);
        };
        /**
         * Deletes a layer by its ID. A certified app can only delete a layer it has created.
         * @param id the id of the layer (if the layer doesn't exist then one will be created)
         * @requires Permissions NAVIGATION_PRIVATE_LAYERS
         * @preview This function is currently available in the Development Environment only.
         */
        this.deleteNavLayer = (id) => {
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NavLayerDeleted, id);
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).deleteNavLayer(id);
        };
        /**
         * Sends a RawRakelAction to the Rakel radio.
         * @param rawRakelAction the RawRakelAction to be sent to the radio.
         * @requires Permissions RAKEL_RAW_COMMAND_SEND
         */
        this.sendRawRakelAction = (rawRakelAction) => {
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.RawRakelActionSent, rawRakelAction);
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).sendRawRakelAction(rawRakelAction);
        };
        if (!EvamApi.singletonExists) {
            const appVersionSetSubscription = (e) => {
                EvamApi.evamData.appVersion = e.detail;
                vsLog("appVersion set", e.detail);
                (0, EventHelpers_1.unsubscribe)(domain_1.EvamEvent.AppVersionSet, appVersionSetSubscription);
            };
            const osVersionSetSubscription = (e) => {
                EvamApi.evamData.osVersion = e.detail;
                vsLog("osVersion set", e.detail);
                (0, EventHelpers_1.unsubscribe)(domain_1.EvamEvent.OSVersionSet, osVersionSetSubscription);
            };
            const vehicleServicesVersionSetSubscription = (e) => {
                EvamApi.evamData.vsVersion = e.detail;
                vsLog("vsVersion set", e.detail);
                (0, EventHelpers_1.unsubscribe)(domain_1.EvamEvent.VehicleServicesVersionSet, vehicleServicesVersionSetSubscription);
            };
            const deviceIdSetSubscription = (e) => {
                EvamApi.evamData.deviceId = e.detail;
                vsLog("deviceId set", e.detail);
                (0, EventHelpers_1.unsubscribe)(domain_1.EvamEvent.DeviceIdSet, deviceIdSetSubscription);
            };
            const appIdSetSubscription = (e) => {
                const getAllItemsFromLocalStorage = (id) => {
                    for (const key in localStorage) {
                        //reg ex pattern
                        const pattern = new RegExp(`^${id}.*`);
                        if (key.match(pattern)) {
                            const value = localStorage.getItem(key);
                            if (EvamApi.persistentStorageMap !== null) {
                                EvamApi.persistentStorageMap.set(key, value);
                            }
                        }
                    }
                };
                const appId = e.detail;
                EvamApi.evamData.appId = appId;
                vsLog("appId set", e.detail);
                getAllItemsFromLocalStorage(appId);
                (0, EventHelpers_1.unsubscribe)(domain_1.EvamEvent.AppIdSet, appIdSetSubscription);
            };
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.VehicleServicesNotificationCallbackTriggered, (e) => {
                const { detail: callbackId } = e;
                if (typeof callbackId === "string") {
                    vsLog("triggerNotificationCallback", e.detail);
                    EvamApi.triggerNotificationCallback(callbackId);
                }
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.AppVersionSet, appVersionSetSubscription);
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.OSVersionSet, osVersionSetSubscription);
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.VehicleServicesVersionSet, vehicleServicesVersionSetSubscription);
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.DeviceIdSet, deviceIdSetSubscription);
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.AppIdSet, appIdSetSubscription);
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedOperation, (e) => {
                const { detail: op } = e;
                vsLog("NewOrUpdatedOperation", op);
                EvamApi.evamData.activeCase = op ? domain_1.Operation.fromJSON(op) : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedSettings, (e) => {
                vsLog("NewOrUpdatedSettings", e.detail);
                EvamApi.evamData.settings = e.detail || undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedInternetState, (e) => {
                const { detail: internetState } = e;
                vsLog("NewOrUpdatedInternetState", internetState);
                EvamApi.evamData.internetState = internetState ? internetState : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedDeviceRole, (e) => {
                const { detail: deviceRole } = e;
                vsLog("NewOrUpdatedDeviceRole", deviceRole);
                EvamApi.evamData.deviceRole = deviceRole ? deviceRole : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedLocation, (e) => {
                const { detail: location } = e;
                vsLog("NewOrUpdatedLocation", location);
                EvamApi.evamData.location = location ? domain_1.Location.fromJSON(location) : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedVehicleState, (e) => {
                const { detail: vehicleState } = e;
                vsLog("NewOrUpdatedVehicleState", vehicleState);
                EvamApi.evamData.vehicleState = vehicleState ? domain_1.VehicleState.fromJSON(vehicleState) : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedTripLocationHistory, (e) => {
                const { detail: tlh } = e;
                vsLog("NewOrUpdatedTripLocationHistory", tlh);
                EvamApi.evamData.tripLocationHistory = tlh ? domain_1.TripLocationHistory.fromJSON(e.detail) : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedOperationList, (e) => {
                const list = e.detail;
                vsLog("NewOrUpdatedOperationList", list);
                if (Array.isArray(list)) {
                    EvamApi.evamData.operationList = list.map(domain_1.Operation.fromJSON);
                }
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedAvailableVehicleStatusList, (e) => {
                const list = e.detail;
                vsLog("NewOrUpdatedAvailableVehicleStatusList", list);
                if (Array.isArray(list)) {
                    EvamApi.evamData.availableVehicleStatusList = list.map(domain_1.VehicleStatus.fromJSON);
                }
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedBattery, (e) => {
                const { detail: battery } = e;
                vsLog("NewOrUpdatedBattery", battery);
                EvamApi.evamData.battery = battery ? domain_1.Battery.fromJSON(battery) : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedDisplayMode, (e) => {
                const { detail: displayMode } = e;
                vsLog("NewOrUpdatedDisplayMode", displayMode);
                EvamApi.evamData.displayMode = displayMode ? displayMode : undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.GRPCEstablished, (e) => {
                const { detail: grpc } = e;
                vsLog("GRPCEstablished", grpc);
                EvamApi.evamData.grpc = grpc || undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdateRakelState, (e) => {
                const { detail: rakelState } = e;
                vsLog("RakelState", rakelState);
                EvamApi.evamData.rakelState = rakelState || undefined;
            });
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedRakelMessages, (e) => {
                const { detail: rakelMessages } = e;
                vsLog("RakelMessages", rakelMessages);
                EvamApi.evamData.rakelMessages = rakelMessages || undefined;
            });
            if (!EvamApi.isRunningInVehicleServices)
                EvamApi.persistentStorageMap = new Map([]);
            EvamApi.singletonExists = true;
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).apiReady();
        }
    }
    /**
     * Sets the selected hospital id for the current active operation. The id must be present inside the available hospitals
     * @requires Permissions ACTIVE_OPERATION_WRITE
     * @param id the id of the hospital to be set
     */
    setHospital(id) {
        var _a, _b;
        if (((_a = EvamApi.evamData.activeCase) === null || _a === void 0 ? void 0 : _a.availableHospitalLocations) === undefined || ((_b = EvamApi.evamData.activeCase) === null || _b === void 0 ? void 0 : _b.availableHospitalLocations.length) === 0) {
            throw Error("Current Operation has no available hospitals.");
        }
        const hl = EvamApi.evamData.activeCase.availableHospitalLocations.find((loc) => {
            return loc.id === id;
        });
        if (hl) {
            if (!EvamApi.isRunningInVehicleServices) {
                const newActiveOperation = lodash_1.default.clone(EvamApi.evamData.activeCase);
                newActiveOperation.selectedHospital = hl.id;
                this.injectOperation(newActiveOperation);
            }
            else {
                (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).setHospital(id);
            }
        }
        else {
            throw Error("Hospital id not located within available hospitals");
        }
    }
    /**
     * Sets the selected priority id for the current active operation. The id must be present inside the available priorities.
     * @requires Permissions ACTIVE_OPERATION_WRITE
     * @param id of the priority to be set
     */
    setPriority(id) {
        if (EvamApi.evamData.activeCase === undefined) {
            throw Error("Can't set priority when there is no active case.");
        }
        else {
            const p = EvamApi.evamData.activeCase.availablePriorities.find((p) => {
                return p.id === id;
            });
            if (p) {
                if (!EvamApi.isRunningInVehicleServices) {
                    const newActiveOperation = lodash_1.default.clone(EvamApi.evamData.activeCase);
                    newActiveOperation.selectedPriority = p.id;
                    this.injectOperation(newActiveOperation);
                }
                else {
                    (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).setPriority(id);
                }
            }
            else {
                throw Error("Cant set priority when priority is not an available priority");
            }
        }
    }
    /**
     * Manually inject location to EvamApi (Only available in development.)
     * @param location the location to inject.
     */
    injectLocation(location) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.location = location;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedLocation, location);
        }
        else {
            throw Error("Injecting an Location is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject the Available Vehicle Status list to EvamApi (Only available in development.)
     * @param vehicleStatusList the list of available Vehicle Statuses
     */
    injectAvailableVehicleStatusList(vehicleStatusList) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.availableVehicleStatusList = vehicleStatusList;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedAvailableVehicleStatusList, vehicleStatusList);
        }
        else {
            throw Error("Injecting an AvailableVehicleStatusList is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject the Rakel State to EvamApi (Only available in development.)
     * @param rakelState The Rakel State
     */
    injectRakelState(rakelState) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.rakelState = rakelState;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdateRakelState, rakelState);
        }
        else {
            throw Error("Injecting a RakelState is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject vehicleState to EvamApi (Only available in development.)
     * @param vehicleState the vehicleState to inject.
     */
    injectVehicleState(vehicleState) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.vehicleState = vehicleState;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedVehicleState, vehicleState);
        }
        else {
            throw Error("Injecting an VehicleState is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject tripLocationHistory to EvamApi (Only available in development.)
     * @param tripLocationHistory the tripLocationHistory to inject.
     */
    injectTrip(tripLocationHistory) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.tripLocationHistory = tripLocationHistory;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedTripLocationHistory, tripLocationHistory);
        }
        else {
            throw Error("Injecting an TripLocationHistory is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject deviceRole to EvamApi (Only available in development.)
     * @param deviceRole the deviceRole to inject.
     */
    injectDeviceRole(deviceRole) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.deviceRole = deviceRole;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedDeviceRole, deviceRole);
        }
        else {
            throw Error("Injecting an DeviceRole is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Manually inject internetState to EvamApi (Only available in development.)
     * @param internetState the internetState to inject.
     */
    injectInternetState(internetState) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.internetState = internetState;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedInternetState, internetState);
        }
        else {
            throw Error("Injecting an internetState is not allowed in the Vehicle Services environment.");
        }
    }
    /**
     * Injects the Active Operation manually. This will trigger onNewOrUpdatedActiveOperation(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param activeCase The active case to be injected for development purposes.
     */
    injectOperation(activeCase) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.activeCase = activeCase;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedOperation, activeCase);
        }
        else {
            throw Error("Injecting an Operation is not allowed in the Vehicle Services environment, use the Vehicle Services Demo tool instead.");
        }
    }
    /**
     * Injects the settings manually. This will trigger onNewOrUpdatedSettings(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param settings The settings to be injected for development purposes.
     */
    injectSettings(settings) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.settings = settings;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedSettings, settings);
        }
        else {
            throw Error("Injecting settings is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    /**
     * Injects the display mode manually. This will trigger onNewOrUpdatedDisplayMode(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param displayMode The display mode (light or dark) to be injected for development purposes.
     */
    injectDisplayMode(displayMode) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.displayMode = displayMode;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedDisplayMode, displayMode);
        }
        else {
            throw Error("Injecting display mode is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    injectAppVersion(appVersion) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.appVersion = appVersion;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.AppVersionSet, appVersion);
        }
        else {
            throw Error("Injecting app version is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    injectOSVersion(osVersion) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.osVersion = osVersion;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.OSVersionSet, osVersion);
        }
        else {
            throw Error("Injecting OS version is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    injectVSVersion(vsVersion) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.vsVersion = vsVersion;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.VehicleServicesVersionSet, vsVersion);
        }
        else {
            throw Error("Injecting VS version is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    injectAppId(appId) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.appId = appId;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.AppIdSet, appId);
        }
        else {
            throw Error("Injecting app id is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    /**
     * Injects the operation list manually. This will trigger onNewOrUpdatedOperationList(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param operationList The operation list to be injected for development purposes.
     */
    injectOperationList(operationList) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.operationList = operationList;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedOperationList, operationList);
        }
        else {
            throw Error("Injecting operation list is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    /**
     * Injects the battery manually. This will trigger onNewOrUpdatedBattery(...)'s callback.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param battery The battery to be injected for development purposes.
     */
    injectBattery(battery) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.battery = battery;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedBattery, battery);
        }
        else {
            throw Error("Injecting battery is not allowed in the Vehicle Services environment, use a web browser instead.");
        }
    }
    /**
     * Inject a list of raw Rakel messages as they would be received from the radio.
     * This function is to be used for development only and will throw an error when used in Vehicle Services.
     * @param rakelMessages list of raw Rakel messages.
     */
    injectRakelMessages(rakelMessages) {
        if (!EvamApi.isRunningInVehicleServices) {
            EvamApi.evamData.rakelMessages = rakelMessages;
            (0, EventHelpers_1.publish)(domain_1.EvamEvent.NewOrUpdatedRakelMessages, rakelMessages);
        }
        else {
            throw Error("Injecting rakel messaged is not allowed in the VS environment, use a web browser instead");
        }
    }
    /**
     * Registers a callback to be run upon a new Active Operation is available or the current Active
     * Operation is updated.
     * @requires Permissions ACTIVE_OPERATION_READ
     * @param callback The callback to be executed
     */
    onNewOrUpdatedActiveOperation(callback) {
        if (callback) {
            const c = (e) => {
                const op = e.detail;
                callback(op ? domain_1.Operation.fromJSON(op) : undefined);
            };
            EvamApi.newOrUpdatedOperationCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedOperation, {
                detail: EvamApi.evamData.activeCase
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedOperation, c);
        }
    }
    /**
     * Registers a callback to be run upon new application settings reception or settings update
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedSettings(callback) {
        if (callback) {
            const c = (e) => {
                const settings = e.detail;
                callback(settings ? settings : undefined);
            };
            EvamApi.newOrUpdatedSettingsCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedSettings, {
                detail: EvamApi.evamData.settings
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedSettings, c);
        }
    }
    /**
     * Registers a callback to be run upon new device role or device role update
     * @requires Permissions DEVICE_ROLE_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedDeviceRole(callback) {
        if (callback) {
            const c = (e) => {
                const deviceRole = e.detail;
                callback(deviceRole ? deviceRole : undefined);
            };
            EvamApi.newOrUpdatedDeviceRoleCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedDeviceRole, {
                detail: EvamApi.evamData.deviceRole
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedDeviceRole, c);
        }
    }
    /**
     * Registers a callback to be run upon new location or location update
     * @requires Permissions LOCATION_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedLocation(callback) {
        if (callback) {
            const c = (e) => {
                const loc = e.detail;
                callback(loc ? domain_1.Location.fromJSON(loc) : undefined);
            };
            EvamApi.newOrUpdatedLocationCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedLocation, {
                detail: EvamApi.evamData.location
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedLocation, c);
        }
    }
    /**
     * Registers a callback to be run upon new internetState or internetState update
     * @requires Permissions CONNECTIVITY_READ
     * @param callback The callback to be executed.
     */
    onNewOrUpdatedInternetState(callback) {
        if (callback) {
            const c = (e) => {
                const internetState = e.detail;
                callback(internetState ? internetState : undefined);
            };
            EvamApi.newOrUpdatedInternetStateCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedInternetState, {
                detail: EvamApi.evamData.internetState
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedInternetState, c);
        }
    }
    /**
     * Used to assign a callback when the vehicle state is updated.
     * @requires Permissions VEHICLE_STATE_READ
     * @param callback The callback with (optional) argument vehicleState. Use this to access the vehicle state.
     */
    onNewOrUpdatedVehicleState(callback) {
        if (callback) {
            const c = (e) => {
                const vehicleState = e.detail;
                callback(vehicleState ? domain_1.VehicleState.fromJSON(vehicleState) : undefined);
            };
            EvamApi.newOrUpdatedVehicleStateCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedVehicleState, {
                detail: EvamApi.evamData.vehicleState
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedVehicleState, c);
        }
    }
    /**
     * Used to assign a callback when the trip location history is updated.
     * @requires Permissions TRIP_HISTORY_READ
     * @param callback The callback with (optional) argument tripLocationHistory. Use this to access the trip location history.
     */
    onNewOrUpdatedTripLocationHistory(callback) {
        if (callback) {
            const c = (e) => {
                const { detail } = e;
                callback(detail ? domain_1.TripLocationHistory.fromJSON(detail) : undefined);
            };
            EvamApi.newOrUpdatedTripLocationHistoryCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedTripLocationHistory, {
                detail: EvamApi.evamData.tripLocationHistory
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedTripLocationHistory, c);
        }
    }
    /**
     * Used to assign a callback when the operation list is updated.
     * @requires Permissions OPERATION_READ
     * @param callback The callback with (optional) argument operationList. Use this to access the operation list.
     */
    onNewOrUpdatedOperationList(callback) {
        if (callback) {
            const c = (e) => {
                const ol = e.detail;
                if (Array.isArray(ol)) {
                    callback(ol.map(domain_1.Operation.fromJSON));
                }
                else if (ol === undefined) {
                    callback(undefined);
                }
            };
            EvamApi.newOrUpdatedOperationListCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedOperationList, {
                detail: EvamApi.evamData.operationList
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedOperationList, c);
        }
    }
    /**
     * Used to assign a callback when the battery data is updated.
     * @requires Permissions BATTERY_READ
     * @param callback The callback with (optional) argument battery. Use this to access the battery.
     */
    onNewOrUpdatedBattery(callback) {
        if (callback) {
            const c = (e) => {
                const battery = e.detail;
                callback(battery ? battery : undefined);
            };
            EvamApi.newOrUpdatedBatteryCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedBattery, {
                detail: EvamApi.evamData.battery
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedBattery, c);
        }
    }
    /**
     *
     * Used to assign a callback when the battery created or updated.
     * @requires Permissions DISPLAY_MODE_READ
     * @param callback The callback with (optional) argument display mode. Use this to access the display mode.
     */
    onNewOrUpdatedDisplayMode(callback) {
        if (callback) {
            const c = (e) => {
                const displayMode = e.detail;
                callback(displayMode ? displayMode : undefined);
            };
            EvamApi.newOrUpdatedDisplayModeCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedDisplayMode, {
                detail: EvamApi.evamData.displayMode
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedDisplayMode, c);
        }
    }
    /**
     * Used to assign a callback when the rakel state is created or updated
     * @requires Permissions RAKEL_STATE_READ
     * @param callback The callback with (optional) argument Rakel state. Use this to access the Rakel state.
     * @preview This function is currently available in the Development Environment only.
     */
    onNewOrUpdatedRakelState(callback) {
        if (callback) {
            const c = (e) => {
                const rakelState = e.detail;
                callback(rakelState ? rakelState : undefined);
            };
            EvamApi.newOrUpdatedRakelStateCallbacks.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdateRakelState, {
                detail: EvamApi.evamData.rakelState
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdateRakelState, c);
        }
    }
    /**
     * Used to assign a callback when the list of available Vehicle Statuses is created or updated
     * @requires Permissions AVAILABLE_VEHICLE_STATUS_LIST_READ
     * @preview This function is currently available in the Development Environment only.
     * @param callback The callback with (optional) argument available Vehicle Status list. Use this to access the available Vehicle Statuses.
     */
    onNewOrUpdatedAvailableVehicleStatusList(callback) {
        if (callback) {
            const c = (e) => {
                const vl = e.detail;
                if (Array.isArray(vl)) {
                    callback(vl.map(domain_1.VehicleStatus.fromJSON));
                }
                else if (vl === undefined) {
                    callback(undefined);
                }
            };
            EvamApi.newOrUpdatedAvailableVehicleStatusList.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedAvailableVehicleStatusList, {
                detail: EvamApi.evamData.availableVehicleStatusList
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedAvailableVehicleStatusList, c);
        }
    }
    /**
     * Used to assign a callback when the incoming Rakel messages are updated.
     * The messages are piped though in the raw form as they are received from the radio.
     * @requires Permissions RAKEL_COMMUNICATION_READ
     * @preview This function is currently available in the Development Environment only.
     * @param callback The callback with (optional) argument Rakel messages. Use this to access the incoming Rakel messages.
     */
    onNewOrUpdatedRakelMessages(callback) {
        if (callback) {
            const c = (e) => {
                const messages = e.detail;
                if (Array.isArray(messages)) {
                    callback(messages);
                    //the reason we check for null here is that dispatching an undefined detail to an event becomes null
                    //this is silly legacy JS stuff
                }
                else if (messages === undefined || messages === null) {
                    callback(undefined);
                }
            };
            EvamApi.newOrUpdatedRakelMessages.push(c);
            c(new CustomEvent(domain_1.EvamEvent.NewOrUpdatedRakelMessages, {
                detail: EvamApi.evamData.rakelMessages
            }));
            (0, EventHelpers_1.subscribe)(domain_1.EvamEvent.NewOrUpdatedRakelMessages, c);
        }
    }
    /**
     * send a notification to vehicle services (or evam-dev-environment if using the dev environment)
     * @requires Permissions SEND_NOTIFICATION
     * @param notification The notification to be sent
     */
    sendNotification(notification) {
        const { heading, description, notificationType, primaryButton, secondaryButton, } = notification;
        let primaryButtonCallbackUUID = undefined;
        let secondaryButtonCallbackUUID = undefined;
        //The UUID used to identify the callback
        //both callbacks have near-identical identifiers except for the last two digits
        //say the callback uuid generated is '12345' then the primary callback id will be '12345-p' whereas the secondary callback
        //will be '12345-s'
        //The reason we do this is that when the callbacks are stored they don't currently store which notification they belong to, but rather this
        //is handled by the notification. We now use this similar callback to identify the notification so that we can clear the memory of both (if both exist)
        //callbacks.
        const callbackUUID = (0, uuid_1.v4)();
        if (primaryButton.callback) {
            //Store the primary button callback in the callbacks Map with an uuid
            primaryButtonCallbackUUID = callbackUUID + "-p";
            EvamApi.notificationCallbacks.set(primaryButtonCallbackUUID, primaryButton.callback);
        }
        //Store the secondary button callback in the callbacks Map with an uuid IF the secondary button is defined
        if ((secondaryButton !== undefined) && (secondaryButton.callback !== undefined)) {
            secondaryButtonCallbackUUID = callbackUUID + "-s";
            EvamApi.notificationCallbacks.set(secondaryButtonCallbackUUID, secondaryButton.callback);
        }
        const vehicleServicesNotificationToSend = {
            heading,
            description,
            notificationType,
            primaryButton: {
                label: primaryButton.label,
                callback: primaryButtonCallbackUUID
            },
            secondaryButton: secondaryButton ? {
                label: secondaryButton.label,
                callback: secondaryButtonCallbackUUID
            } : undefined
        };
        if (EvamApi.isRunningInVehicleServices) {
            (0, AndroidNativeHelpers_1.androidNativeHelpers)(EvamApi.isRunningInVehicleServices).sendNotification(vehicleServicesNotificationToSend);
        }
        (0, EventHelpers_1.publish)(domain_1.EvamEvent.VehicleServicesNotificationSent, vehicleServicesNotificationToSend);
    }
}
exports.EvamApi = EvamApi;
EvamApi.singletonExists = false;
EvamApi.isRunningInVehicleServices = AndroidNativeHelpers_1.isRunningInVehicleServices;
/**
 * EvamData instance for storing data regard Vehicle Services.
 * @private
 */
EvamApi.evamData = new EvamData();
/**
 * These arrays store references to every callback subscribed to certain events. We use these references to later unsubscribe from DOM events.
 * @hidden
 */
EvamApi.newOrUpdatedOperationCallbacks = [];
EvamApi.newOrUpdatedSettingsCallbacks = [];
EvamApi.newOrUpdatedLocationCallbacks = [];
EvamApi.newOrUpdatedDeviceRoleCallbacks = [];
EvamApi.newOrUpdatedInternetStateCallbacks = [];
EvamApi.newOrUpdatedVehicleStateCallbacks = [];
EvamApi.newOrUpdatedTripLocationHistoryCallbacks = [];
EvamApi.newOrUpdatedOperationListCallbacks = [];
EvamApi.newOrUpdatedBatteryCallbacks = [];
EvamApi.newOrUpdatedDisplayModeCallbacks = [];
EvamApi.newOrUpdatedRakelStateCallbacks = [];
EvamApi.newOrUpdatedAvailableVehicleStatusList = [];
EvamApi.newOrUpdatedRakelMessages = [];
EvamApi.notificationCallbacks = new Map([]);
/**
 * The persistentStorageMap object is only used for testing. It is null unless the isRunningInVehicleServices method
 * returns false.
 */
EvamApi.persistentStorageMap = null;
EvamApi.triggerNotificationCallback = (uuid) => {
    const callback = EvamApi.notificationCallbacks.get(uuid);
    if (callback) {
        callback();
        EvamApi.notificationCallbacks.delete(uuid);
        //This deletes the other callback (so if we called secondary then primary is deleted from memory and vice versa)
        const isPrimaryCallback = uuid.slice(-2) === "-p"; //Primary callback UUIDs end with "-p" whereas secondary end with "-s"
        const correspondingCallbackTypeIdentifier = isPrimaryCallback ? "-s" : "-p"; //which kind of callback aren't we deleting (the one that wasn't called)
        const correspondingCallbackUUID = uuid.slice(0, -2) + correspondingCallbackTypeIdentifier; //the UUID will be the same as the triggered callback but with different last two chars
        const correspondingCallback = EvamApi.notificationCallbacks.get(correspondingCallbackUUID); //Find the callback
        if (correspondingCallback) { //If it exists then delete it
            EvamApi.notificationCallbacks.delete(correspondingCallbackUUID);
        }
    }
};
//# sourceMappingURL=EvamApi.js.map