"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Operation = void 0;
const VehicleStatus_1 = require("./VehicleStatus");
const DestinationSiteLocation_1 = require("./DestinationSiteLocation");
const DestinationControlPointLocation_1 = require("./DestinationControlPointLocation");
const HospitalLocation_1 = require("./HospitalLocation");
const OperationPriority_1 = require("./OperationPriority");
const LeavePatientLocation_1 = require("./LeavePatientLocation");
class Operation {
    /**
     * @param operationID The operation ID
     * @param name The operation name
     * @param sendTime The time at which the operation was sent
     * @param createdTime The time at which the operation was created
     * @param acceptedTime The time at which the operation was accepted (PREVIEW)
     * @param endTime The time at which the operation ended, `undefined` still ongoing
     * @param callCenterId The Call Center ID
     * @param caseFolderId The Case folder ID
     * @param transmitterCode The transmitter code
     * @param alarmCategory The alarm category
     * @param alarmEventCode The alarm event code
     * @param medicalCommander The medical commander of this operation
     * @param medicalIncidentOfficer The medical incident office of this operation
     * @param attachedCustomerObject The attached customer Object
     * @param alarmEventText The alarm event text
     * @param additionalInfo The additional information attached to this operation
     * @param keyNumber The key number
     * @param electronicKey The electronic key
     * @param radioGroupMain The main radio group associated with this operation
     * @param radioGroupSecondary The secondary radio group associated with this operation
     * @param additionalCoordinationInformation Additional coordination information
     * @param availablePriorities The list of available priorities to select from
     * @param patientName The name of the patient if any
     * @param patientUID The personal number of the patient if any
     * @param vehicleStatus The current vehicle status in this operation
     * @param destinationSiteLocation The location of the destination
     * @param breakpointLocation The location of the breakpoint if any
     * @param availableHospitalLocations The list of available hospitals to select from
     * @param header1 The case index 2
     * @param header2 The case index 3
     * @param eventInfo The event description
     * @param caseInfo The case info comment
     * @param selectedHospital The id of the selected hospital [inside available hospitals]
     * @param selectedPriority The id of the selected priority [inside available priority]
     * @param operationState the current state of the operation (ACTIVE, AVAILABLE, COMPLETE)
     * @param leavePatientLocation The location of the leave-patient ('toCity', etc)
     */
    constructor(
    // Metadata
    operationID, name, sendTime, createdTime, endTime, acceptedTime, 
    // Rakel
    callCenterId, caseFolderId, 
    // Misc
    transmitterCode, alarmCategory, alarmEventCode, medicalCommander, // incidentCommander
    medicalIncidentOfficer, attachedCustomerObject, alarmEventText, additionalInfo, keyNumber, electronicKey, 
    // Radio groups
    radioGroupMain, radioGroupSecondary, 
    // Extra
    additionalCoordinationInformation, 
    // Priority
    availablePriorities, 
    // Patient
    patientName, patientUID, 
    // status, destinations
    vehicleStatus, destinationSiteLocation, breakpointLocation, availableHospitalLocations, 
    // Case index 2, 3, event description, comment
    header1, header2, eventInfo, caseInfo, selectedHospital, selectedPriority, operationState, leavePatientLocation) {
        this.operationID = operationID;
        this.name = name;
        this.sendTime = sendTime;
        this.createdTime = createdTime;
        this.endTime = endTime;
        this.acceptedTime = acceptedTime;
        this.callCenterId = callCenterId;
        this.caseFolderId = caseFolderId;
        this.transmitterCode = transmitterCode;
        this.alarmCategory = alarmCategory;
        this.alarmEventCode = alarmEventCode;
        this.medicalCommander = medicalCommander;
        this.medicalIncidentOfficer = medicalIncidentOfficer;
        this.attachedCustomerObject = attachedCustomerObject;
        this.alarmEventText = alarmEventText;
        this.additionalInfo = additionalInfo;
        this.keyNumber = keyNumber;
        this.electronicKey = electronicKey;
        this.radioGroupMain = radioGroupMain;
        this.radioGroupSecondary = radioGroupSecondary;
        this.additionalCoordinationInformation = additionalCoordinationInformation;
        this.availablePriorities = availablePriorities;
        this.patientName = patientName;
        this.patientUID = patientUID;
        this.vehicleStatus = vehicleStatus;
        this.destinationSiteLocation = destinationSiteLocation;
        this.breakpointLocation = breakpointLocation;
        this.availableHospitalLocations = availableHospitalLocations;
        this.header1 = header1;
        this.header2 = header2;
        this.eventInfo = eventInfo;
        this.caseInfo = caseInfo;
        this.selectedHospital = selectedHospital;
        this.selectedPriority = selectedPriority;
        this.operationState = operationState;
        this.leavePatientLocation = leavePatientLocation;
    }
    /**
     * Gets the full operation ID, e.g. '1:18:6546', composed by 'callCenterId:caseFolderId:operationID'.
     * @returns id The full operation ID
     */
    getFullId() {
        return `${this.callCenterId}:${this.caseFolderId}:${this.operationID}`;
    }
    /**
     * Create from JSON
     * @param data JSON object
     */
    static fromJSON(data) {
        if (data.operationID === undefined || data.name === undefined || data.operationState === undefined) {
            throw Error("Operation Id, Operation Name and Operation State must be specified for Operation");
        }
        return new Operation(data.operationID, data.name, data.sendTime !== undefined ? new Date(data.sendTime) : undefined, data.createdTime !== undefined ? new Date(data.createdTime) : undefined, data.endTime !== undefined ? new Date(data.endTime) : undefined, data.acceptedTime !== undefined ? new Date(data.acceptedTime) : undefined, data.callCenterId, data.caseFolderId, data.transmitterCode, data.alarmCategory, data.alarmEventCode, data.medicalCommander, data.medicalIncidentOfficer, data.attachedCustomerObject, data.alarmEventText, data.additionalInfo, data.keyNumber, data.electronicKey, data.radioGroupMain, data.radioGroupSecondary, data.additionalCoordinationInformation, (data.availablePriorities !== undefined && Array.isArray(data.availablePriorities)) ? data.availablePriorities.map((ap) => (OperationPriority_1.OperationPriority.fromJSON(ap))) : undefined, data.patientName, data.patientUID, data.vehicleStatus !== undefined ? VehicleStatus_1.VehicleStatus.fromJSON(data.vehicleStatus) : undefined, data.destinationSiteLocation !== undefined ? DestinationSiteLocation_1.DestinationSiteLocation.fromJSON(data.destinationSiteLocation) : undefined, data.destinationControlPointLocation !== undefined ? DestinationControlPointLocation_1.DestinationControlPointLocation.fromJSON(data.destinationControlPointLocation) : undefined, (data.availableHospitalLocations !== undefined && Array.isArray(data.availableHospitalLocations)) ? data.availableHospitalLocations.map((hl) => HospitalLocation_1.HospitalLocation.fromJSON(hl)) : undefined, data.header1, data.header2, data.eventInfo, data.caseInfo, data.selectedHospital, data.selectedPriority, data.operationState, data.leavePatientLocation !== undefined ? LeavePatientLocation_1.LeavePatientLocation.fromJSON(data.leavePatientLocation) : undefined);
    }
}
exports.Operation = Operation;
//# sourceMappingURL=Operation.js.map